import re
from selenium.webdriver.remote.webdriver import WebDriver
from typing import TypedDict, Callable, Sequence
from . import portal_list
from .util import wait_url_change

TRY_PORTAL_URL = "https://www.google.com/"


def try_login(driver: WebDriver) -> None:
    url = driver.current_url
    driver.get(TRY_PORTAL_URL)
    url = wait_url_change(url, driver)
    if url == TRY_PORTAL_URL:
        print("Already connected")
        return

    for p in portal_list.portals:
        if re.search(p["pattern"], url) is not None:
            print(
                f"Known portal found:\n\tUrl: {url}\n\tMatch: {p['pattern']}\nAttempting to login..."
            )
            p["login"](driver)
            return

    print(f"Unknown portal found:\n\tUrl: {url}\n\tMatch: None")



from selenium.webdriver.remote.webdriver import WebDriver
from selenium.common.exceptions import NoSuchElementException

from .util import wait_url_change


def login(driver: WebDriver):
    url = driver.current_url
    try:
        print(f"Find button_next_page")
        button_next_page = driver.find_element_by_id("button_next_page")
        button_next_page.click()
        url = wait_url_change(url, driver)
        print(f"Url: {url}")
    except NoSuchElementException:
        print(f"Not found: button_next_page")

    try:
        print(f"Find button_accept")
        button_accept = driver.find_element_by_id("button_accept")
        button_accept.click()
        url = wait_url_change(url, driver)
        print(f"Url: {url}")
    except NoSuchElementException:
        print(f"Not found: button_accept")



from selenium.webdriver.remote.webdriver import WebDriver
from typing import TypedDict, Callable, Sequence
from . import portal_wi2


class Portal(TypedDict):  # pylint: disable=inherit-non-class
    pattern: str
    login: Callable[[WebDriver], None]


# A List mapping portals to their login scipts
portals: Sequence[Portal] = [
    {"pattern": r"service\.wi2\.ne\.jp", "login": portal_wi2.login}
]



import cafelogin
import sys
import requests
import configargparse
import cafelogin.actions as actions

DEFAULT_CONFIG_FILES = ["cafelogin.config", "~/cafelogin.config"]


def add_help_argument(parser: configargparse.ArgumentParser):
    parser.add_argument(
        "-h",
        "--help",
        action="help",
        help="Show this help message.",
    )


def add_config_argument(parser: configargparse.ArgumentParser):
    parser.add_argument(
        "-c",
        "--config-file",
        is_config_file=True,
        help=f"Args that start with '--' (eg. --driver-version) can also be set in a config file ({', '.join(DEFAULT_CONFIG_FILES)} or specified via -c). "
        + "Config file syntax allows: key=value, flag=true, stuff=[a,b,c] (details here https://goo.gl/R74nmi). "
        + "Arg precedence: commandline > config-file > defaults.",
    )


def run():
    parser = configargparse.ArgParser(
        default_config_files=DEFAULT_CONFIG_FILES,
        formatter_class=configargparse.ArgumentDefaultsHelpFormatter,
        add_help=False,
        add_config_file_help=False,
    )
    add_help_argument(parser)
    add_config_argument(parser)
    parser.add_argument(
        "--version",
        action="version",
        version=cafelogin.__version__,
    )
    parser.add_argument(
        "--driver-version",
        default=actions.DEFAULT_DRIVER_VERSION,
        help="Specify the webdriver version to use.",
    )
    parser.add_argument(
        "--watch",
        action="store_true",
        help="Continue watching for changes in portal connectivity.",
    )
    parser.add_argument(
        "--watch-interval",
        type=int,
        default=20,
        help="The frequency with which to watch for portal changes.",
    )

    args, unused_args = parser.parse_known_args()

    try:
        with actions.create_webdriver_context(
            driver_version=args.driver_version
        ) as driver:
            if args.watch:
                actions.watch_portal_connection(
                    driver=driver, watch_interval=args.watch_interval
                )
            else:
                actions.ensure_portal_connection(driver=driver)
    except requests.exceptions.ConnectionError as error:
        sys.exit(error)
    except KeyboardInterrupt:
        pass



from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions


def wait_url_change(url: str, driver: WebDriver):
    wait = WebDriverWait(driver, 3)
    wait.until(expected_conditions.url_changes(url))
    url = driver.current_url
    return url



import colorama


def clr(*colored_text: str, sep: str = "") -> str:
    """Add the color reset code after each colored_text."""
    return (sep + colorama.Style.RESET_ALL).join(
        colored_text
    ) + colorama.Style.RESET_ALL


def print_clr(*colored_text: str, sep: str = ""):
    """Add the color reset code after each colored_text."""
    print(clr(*colored_text, sep=sep))



import requests
import time
from datetime import datetime
from contextlib import contextmanager
from selenium import webdriver
from selenium.webdriver.remote.webdriver import WebDriver
from webdriver_manager.firefox import GeckoDriverManager
from typing import Sequence
from .portals import login
from .print_util import print_clr
from colorama import Style, Fore

DETECT_PORTAL_URL = "http://detectportal.firefox.com/"
DEFAULT_DRIVER_VERSION = "v0.29.0"


def install_webdriver(version: str = DEFAULT_DRIVER_VERSION) -> str:
    return GeckoDriverManager(
        version=version,
        cache_valid_range=100,
        print_first_line=False,
    ).install()


@contextmanager
def create_webdriver_context(driver_version: str = DEFAULT_DRIVER_VERSION):
    options = webdriver.firefox.options.Options()
    options.headless = True
    with webdriver.Firefox(
        options=options,
        executable_path=install_webdriver(version=driver_version),
    ) as d:
        yield d


def portal_connected():
    response = requests.get(DETECT_PORTAL_URL)
    return response.ok and response.text.strip() == "success"


def print_portal_status(connected: bool):
    if connected:
        print_clr(
            f"{Fore.GREEN}{datetime.now()}",
            f"{Fore.GREEN}{Style.DIM} - Portal connection up",
        )
    else:
        print_clr(
            f"{Fore.YELLOW}{datetime.now()}",
            f"{Fore.YELLOW}{Style.DIM} - Portal connection down",
        )


def portal_connected_print_if_changed(previous_state: bool):
    current_state = portal_connected()
    if current_state != current_state:
        print_portal_status(current_state)
    return current_state


def ensure_portal_connection(driver: WebDriver):
    if portal_connected():
        print("Already connected")
        return

    login.try_login(driver=driver)

    if portal_connected():
        print("Login succeeded")
    else:
        print("Login failed")


def watch_portal_connection(driver: WebDriver, watch_interval: float):
    print_clr(
        f"{Style.BRIGHT}Checking portal every ",
        f"{Style.BRIGHT}{Fore.CYAN}{watch_interval} ",
        f"{Style.BRIGHT}seconds.  Ctrl-c to exit.",
    )

    connected = portal_connected()
    print_portal_status(connected)

    while True:
        connected = portal_connected_print_if_changed(connected)

        if not connected:
            login.try_login(driver=driver)
            connected = portal_connected_print_if_changed(connected)
            if connected:
                print("Login succeeded")
            else:
                print("Login failed.  Exiting.")
                return
        time.sleep(watch_interval)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cafelogin', 'cafelogin.portals']

package_data = \
{'': ['*']}

install_requires = \
['ConfigArgParse>=1.2.3,<2.0.0',
 'colorama>=0.4.4,<0.5.0',
 'selenium>=3.141.0,<4.0.0',
 'webdriver-manager>=3.3.0,<4.0.0']

entry_points = \
{'console_scripts': ['cafelogin = cafelogin:run']}

setup_kwargs = {
    'name': 'cafelogin',
    'version': '0.1.5',
    'description': 'Login to wifi portals commonly found at cafes.',
    'long_description': '# cafelogin\n\nA python command line tool for logging into cafe wifi portals.\n\n### Compatibility:\n- Browser WebDriver\n  - Firefox (Geckodriver)\n- Wifi Portal\n  - https://service.wi2.ne.jp\n    - Starbucks JP\n    - Wired Cafe JP\n    - Doutor JP\n\n## Install\n\n```zsh\npip install cafelogin\n```\n\n## Usage\n\n```zsh\ncafelogin [-h] [-c CONFIG_FILE] [--driver-version DRIVER_VERSION] [--watch] [--watch-interval WATCH_INTERVAL]```\n```\n\nExamples:\n\n```zsh\n# Check portal connection and login via any detected portal\ncafelogin\n\n# Watch the portal connection continuously for changes\ncafelogin --watch\n\n# Specify a web-driver version to use\ncafelogin --driver-version "v0.28.0"\n```\n\n## WebDriver cache\n\nRun the command once with an internet connection to install the web-driver to the cache.\n\n```\ncafelogin\n\n[WDM] - ====== WebDriver manager ======\n[WDM] - There is no [linux64] geckodriver for browser  in cache\n[WDM] - Getting latest mozilla release info for v0.29.0\n[WDM] - Trying to download new driver from https://github.com/mozilla/geckodriver/releases/download/v0.29.0/geckodriver-v0.29.0-linux64.tar.gz\n[WDM] - Driver has been saved in cache [/home/me/.wdm/drivers/geckodriver/linux64/v0.29.0]\n```\n',
    'author': 'Ben Nordstrom',
    'author_email': 'bennord@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bennord/cafelogin',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
